﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Xml;

namespace MessageDiagnosticsInterceptors
{
    public class CustomServiceHostFactory : ServiceHostFactory
    {
        protected override ServiceHost CreateServiceHost( Type serviceType, Uri[] baseAddresses )
        {
            return new CustomServiceHost( serviceType, baseAddresses );
        }
    }

    public class MessageDiagnosticsInterceptor : IClientMessageInspector, IDispatchMessageInspector
    {
        public object BeforeSendRequest( ref Message request, IClientChannel channel )
        {
            return null;
        }

        public void AfterReceiveReply( ref Message reply, object correlationState ) {}

        public object AfterReceiveRequest( ref Message request, IClientChannel channel, InstanceContext instanceContext )
        {
            return null;
        }

        public void BeforeSendReply( ref Message reply, object correlationState )
        {
            try
            {
                if ( !reply.IsFault && !reply.IsEmpty )
                {
                    if ( !string.IsNullOrEmpty( reply.Headers.Action ) )
                    {
                        XmlDictionaryReader messageBodyReader = reply.GetReaderAtBodyContents();
                        string messageBody = messageBodyReader.ReadOuterXml();

                        DetermineAndLogMessageDiagnostics( messageBody );
                        RebuildMessage( ref reply, messageBody );
                    }
                }
            }
            catch ( Exception e )
            {
                Debug.WriteLine( e );
            }
        }

        private void DetermineAndLogMessageDiagnostics( string messageBody )
        {
            double bodySizeInBytes = Encoding.UTF8.GetByteCount( messageBody );

            double asKiloBytes = bodySizeInBytes / 1024;

            Debug.WriteLine( "Message size in KBytes {0}", asKiloBytes );
        }

        private static void RebuildMessage( ref Message message, string messageBody )
        {
            if ( message.IsFault || message.IsEmpty ) return;

            var bodyDoc = new XmlDocument();
            bodyDoc.LoadXml( messageBody );

            // Create new message 
            Message replacementMessage = Message.CreateMessage( message.Version,
                                                                null,
                                                                new XmlNodeReader( bodyDoc.DocumentElement ) );

            replacementMessage.Headers.CopyHeadersFrom( message );

            foreach ( string propertyKey in message.Properties.Keys )
            {
                replacementMessage.Properties.Add( propertyKey, message.Properties[ propertyKey ] );
            }

            // Close the original message and return new message 
            message.Close();
            message = replacementMessage;
        }
    }
}